import Prism from 'prismjs';
import { Record } from 'immutable';
import { createElement } from 'react';

const TOKEN_MARK = 'prism-token';

function defaultOnlyIn(node) {
    return node.object === 'block' && node.type === 'code_block';
}
function defaultGetSyntax(node) {
    return 'javascript';
}
function defaultRenderMark(props) {
    const { mark } = props;
    if (mark.type !== TOKEN_MARK) {
        return undefined;
    }
    const className = mark.data.get('className');
    return createElement("span", { className: className }, props.children);
}
class Options extends Record({
    onlyIn: defaultOnlyIn,
    getSyntax: defaultGetSyntax,
    renderMark: defaultRenderMark
}) {
}

function PrismPlugin(optsParam = {}) {
    const opts = new Options(optsParam);
    return {
        decorateNode: (node) => {
            if (!opts.onlyIn(node)) {
                return undefined;
            }
            return decorateNode(opts, node);
        },
        renderMark: opts.renderMark,
        TOKEN_MARK
    };
}
function decorateNode(opts, block) {
    const grammarName = opts.getSyntax(block);
    const grammar = Prism.languages[grammarName];
    if (!grammar) {
        return [];
    }
    const texts = block.getTexts();
    const blockText = texts.map(t => t.text).join('\n');
    const tokens = Prism.tokenize(blockText, grammar);
    const decorations = [];
    let textStart = 0;
    let textEnd = 0;
    texts.forEach(text => {
        textEnd = textStart + text.text.length;
        let offset = 0;
        function processToken(token, accu) {
            accu = accu || '';
            if (typeof token === 'string') {
                if (accu) {
                    const decoration = createDecoration({
                        text,
                        textStart,
                        textEnd,
                        start: offset,
                        end: offset + token.length,
                        className: `prism-token token ${accu}`
                    });
                    if (decoration) {
                        decorations.push(decoration);
                    }
                }
                offset += token.length;
            }
            else {
                accu = `${accu} ${token.type} ${token.alias || ''}`;
                if (typeof token.content === 'string') {
                    const decoration = createDecoration({
                        text,
                        textStart,
                        textEnd,
                        start: offset,
                        end: offset + token.content.length,
                        className: `prism-token token ${accu}`
                    });
                    if (decoration) {
                        decorations.push(decoration);
                    }
                    offset += token.content.length;
                }
                else {
                    for (const item of token.content) {
                        processToken(item, accu);
                    }
                }
            }
        }
        tokens.forEach(processToken);
        textStart = textEnd + 1;
    });
    return decorations;
}
function createDecoration({ text, textStart, textEnd, start, end, className }) {
    if (start >= textEnd || end <= textStart) {
        return null;
    }
    start = Math.max(start, textStart);
    end = Math.min(end, textEnd);
    start -= textStart;
    end -= textStart;
    return {
        anchorKey: text.key,
        anchorOffset: start,
        focusKey: text.key,
        focusOffset: end,
        marks: [{ type: 'prism-token', data: { className } }]
    };
}

export default PrismPlugin;
//# sourceMappingURL=index.js.map
